# $Id: chess.tcl,v 1.9 2002/10/15 19:42:54 alexey Exp $

namespace eval chess {
    set square_size 48
    set line_width 1
    variable theme
    if {![info exist theme]} {
	set theme 0
    }
    array set piece_name [list \
			      wk \u2654 \
			      wq \u2655 \
			      wr \u2656 \
			      wb \u2657 \
			      wn \u2658 \
			      wp {} \
			      bk \u265a \
			      bq \u265b \
			      br \u265c \
			      bb \u265d \
			      bn \u265e \
			      bp {}]
    array set prom_name [list q queen r rook b bishop n knight p pawn]
    array set prom_name_r [list queen q rook r bishop b knight n]
}

proc chess::invite_dialog {jid} {
    set w .chess_invite

    if {[winfo exists $w]} {
	destroy $w
    }

    Dialog $w -title "Let's play chess, $jid!" \
	-separator 1 -anchor e -default 0

    # TODO

    $w add -text "Invite" -command [list [namespace current]::invite $jid]
    $w add -text Cancel -command [list destroy $w]

    $w draw
}

proc chess::invite {jid} {
    destroy .chess_invite

    set id chess[random 1000000000]

    # FIX
    set rjid [get_jid_of_user $jid]

    jlib::send_iq set \
	[jlib::wrapper:createtag create \
	     -vars [list xmlns games:board type chess id $id]] \
	-to $rjid -command [list [namespace current]::invite_res $rjid $id]
}

proc chess::invite_res {jid id res child} {
    if {![cequal $res OK]} {
	after idle MessageDlg .chess_invite_error -icon error \
	    -message [list "Opponent $jid refuse our request: $child"] \
	    -type ok
	return ""
    }

    start_play $jid $id white

}





proc chess::invited_dialog {jid id} {
    set w .chess_invited

    if {[winfo exists $w]} {
	destroy $w
    }

    Dialog $w -title "$jid want to play chess!" \
	-separator 1 -anchor e -default 0

    # TODO

    $w add -text "Play"
    $w add -text Cancel -command [list destroy $w]

    if {[$w draw] == 0} {
	destroy .chess_invited
	[namespace current]::reply_ok $jid $id black

	return [jlib::wrapper:createtag create \
		    -vars [list xmlns games:board type chess id $id]]
    } else {
	return ""
    }
}

proc chess::reply_ok {jid id color} {
    start_play $jid $id $color
}


proc chess::start_play {jid id color} {
    variable flags

    set gid [make_gid $jid $id]

    set flags($gid,opponent) $jid
    set flags($gid,id) $id
    set flags($gid,our_color) $color
    set flags($gid,color_on_move) white
    # <color>r<king|queen>_legal - <color> <king|queen>'s side castling is legal
    set flags($gid,wck_legal) 1
    set flags($gid,wcq_legal) 1
    set flags($gid,bck_legal) 1
    set flags($gid,bcq_legal) 1
    set flags($gid,enpassant) {}
    open $gid
}

proc chess::make_gid {jid id} {
    jid_to_tag [concat $jid $id]
}

proc chess::turn_recv {jid id childs} {
    variable flags
    variable prom_name_r

    set gid [make_gid $jid $id]

    foreach child $childs {
	jlib::wrapper:splitxml $child tag vars isempty chdata children
	switch -- $tag {
	    move {
		set pos [jlib::wrapper:getattr $vars pos]
		set poss [split $pos ";"]
		if {[llength $poss] == 2} {
		    set pos1 [split [lindex $poss 0] ,]
		    set pos2 [split [lindex $poss 1] ,]
		    if {[llength $pos1] == 2 && [llength $pos2] == 2} {
			set cf [lindex $pos1 0]
			set rf [lindex $pos1 1]
			set ct [lindex $pos2 0]
			set rt [lindex $pos2 1]
			set prom ""
			foreach child2 $children {
			    jlib::wrapper:splitxml $child2 tag2 vars2 isempty2 chdata2 children2
			    if {$tag2 == "promotion"} {
				set n [array names prom_name_r *]
				if {[lsearch -exact $n $chdata2] != -1} {
				    set prom $prom_name_r($chdata2)
				}
			    }
			}
			do_move $gid "" $cf $rf $ct $rt $prom
			if {$flags($gid,color_on_move) == $flags($gid,our_color)} {
			    $flags($gid,board) config -cursor ""
			} else {
			    $flags($gid,board) config -cursor watch
			}
			draw_position $gid $flags($gid,board)
			return [jlib::wrapper:createtag turn \
				    -vars [list xmlns games:board \
					       type chess \
					       id $flags($gid,id)]]
		    }
		}
	    }
	}
    }

}


###############################################################################

proc chess::load_pixmaps {} {
    variable pixmap
    variable theme

    set path [file join [file dirname [info script]] pixmaps fig$theme]
    foreach f {b k n p q r} {
	set pixmap(b$f) [Bitmap::get [file join $path b$f.xpm]]
	set pixmap(w$f) [Bitmap::get [file join $path w$f.xpm]]
    }
    set pixmap(bf) [Bitmap::get [file join $path bf.xpm]]
    set pixmap(wf) [Bitmap::get [file join $path wf.xpm]]
}

proc chess::calc_moves {} {
    variable moves

    for {set c 0} {$c < 8} {incr c} {
	for {set r 0} {$r < 8} {incr r} {
	    for {set moves(d1,$c,$r) {}; set x [expr $c+1]; set y [expr $r+1]} \
		{($x < 8) && ($y < 8)} {incr x; incr y} {
		lappend moves(d1,$c,$r) $x $y
	    }
	    for {set moves(d2,$c,$r) {}; set x [expr $c-1]; set y [expr $r+1]} \
		{($x >= 0) && ($y < 8)} {incr x -1; incr y} {
		lappend moves(d2,$c,$r) $x $y
	    }
	    for {set moves(d3,$c,$r) {}; set x [expr $c-1]; set y [expr $r-1]} \
		{($x >= 0) && ($y >= 0)} {incr x -1; incr y -1} {
		lappend moves(d3,$c,$r) $x $y
	    }
	    for {set moves(d4,$c,$r) {}; set x [expr $c+1]; set y [expr $r-1]} \
		{($x < 8) && ($y >= 0)} {incr x; incr y -1} {
		lappend moves(d4,$c,$r) $x $y
	    }
	    for {set moves(h1,$c,$r) {}; set x [expr $c+1]} {$x < 8} {incr x} {
		lappend moves(h1,$c,$r) $x $r
	    }
	    for {set moves(h2,$c,$r) {}; set x [expr $c-1]} {$x >= 0} {incr x -1} {
		lappend moves(h2,$c,$r) $x $r
	    }
	    for {set moves(v1,$c,$r) {}; set y [expr $r+1]} {$y < 8} {incr y} {
		lappend moves(v1,$c,$r) $c $y
	    }
	    for {set moves(v2,$c,$r) {}; set y [expr $r-1]} {$y >= 0} {incr y -1} {
		lappend moves(v2,$c,$r) $c $y
	    }
	    set moves(n,$c,$r) {}
	    foreach {dx dy} {1 2 -1 2 -2 1 -2 -1 -1 -2 1 -2 2 -1 2 1} {
	    	set x [expr $c + $dx]
	    	set y [expr $r + $dy]
		if {($x >= 0) && ($x < 8) && ($y >= 0) && ($y < 8)} {
		    lappend moves(n,$c,$r) $x $y
		}
	    }
	    set moves(wpt,$c,$r) {}
	    if {$r <= 6} {
		if {$c <= 6} {
		    lappend moves(wpt,$c,$r) [expr $c + 1] [expr $r + 1]
		}
		if {$c >= 1} {
		    lappend moves(wpt,$c,$r) [expr $c - 1] [expr $r + 1]
		}
	    }
	    set moves(bpt,$c,$r) {}
	    if {$r >= 1} {
		if {$c <= 6} {
		    lappend moves(bpt,$c,$r) [expr $c + 1] [expr $r - 1]
		}
		if {$c >= 1} {
		    lappend moves(bpt,$c,$r) [expr $c - 1] [expr $r - 1]
		}
	    }
	    set moves(k,$c,$r) {}
	    foreach {dx dy} {0 1 -1 1 -1 0 -1 -1 0 -1 1 -1 1 0 1 1} {
		set x [expr $c + $dx]
		set y [expr $r + $dy]
		if {($x >= 0) && ($x < 8) && ($y >= 0) && ($y < 8)} {
		    lappend moves(k,$c,$r) $x $y
		}
	    }
	}
    }
}

proc chess::center {c r} {
    variable square_size
    variable line_width

    set r [expr {7 - $r}]
    list [expr {$line_width + ($square_size * 0.5) + \
		    (($square_size + $line_width) * $c)}] \
	[expr {$line_width + ($square_size * 0.5) + \
		   (($square_size + $line_width) * $r)}]
}

proc chess::open {gid} {
    global font
    variable square_size
    variable line_width
    variable pixmap
    variable flags
    variable piece_name

    set jid $flags($gid,opponent)

    set w [win_id chess $gid]
    if {[winfo exists $w]} {
	return
    }

    add_win $w -title "Chess with $jid" -tabtitle "Chess with $jid" \
	-class Chess

    set board [canvas $w.board \
		   -width [expr {($square_size + $line_width) * 8}] \
		   -height [expr {($square_size + $line_width) * 8}]]
    pack $board -side left -anchor w -padx 10

    set flags($gid,board) $board

    checkbutton $w.allow_illegal -text "Allow illegal moves" \
	-variable [namespace current]::allow_illegal
    pack $w.allow_illegal -side top -anchor w

    frame $w.move 
    pack $w.move -side top -anchor w
    label $w.move.title -text "Move: "
    pack $w.move.title -side left
    label $w.move.on_move \
	-textvariable [namespace current]::flags($gid,color_on_move)
    pack $w.move.on_move -side left -anchor w

#    set htf [TitleFrame $w.htf -text "History"]
#    pack $htf -side top -fill x
#    set hf [$htf getframe]
    label $w.history -text "History"
    pack $w.history -side top -anchor w
    set tabstop1 [font measure $font "99.."]
    set tabstop2 [font measure $font "99..$piece_name(wq)a8-a8+ "]
#    set ht [text $hf.text -font $font -tabs "$tabstop1 $tabstop2"]
    set ht [text $w.text -font $font -tabs "$tabstop1 $tabstop2"]
#    pack $ht -fill both
    pack $ht -side top -fill both
    set flags($gid,hw) $ht

    set dsq_color #77a26d
    set lsq_color #c8c365

    for {set c 0} {$c < 8} {incr c} {
	for {set r 0} {$r < 8} {incr r} {
	    set x1 [expr {$line_width + (($square_size + $line_width) * $c)}]
	    set x2 [expr {($square_size + $line_width) * ($c + 1)}]
	    set y1 [expr {$line_width + (($square_size + $line_width) * $r)}]
	    set y2 [expr {($square_size + $line_width) * ($r + 1)}]
	    set color [expr {($c+$r) % 2 ? $dsq_color : $lsq_color}]
	    set img [expr {($c+$r) % 2 ? "bf" : "wf"}]

	    $board create image $x1 $y1 -image $pixmap($img) -anchor nw \
		-tags [list background [list cr $c [expr {7-$r}]]]
	    $board create rectangle $x1 $y1 $x2 $y2 \
		-outline {} \
		-tags [list square [list cr $c [expr {7-$r}]]]
	}
    }

    $board bind figure <1> \
	[list [namespace current]::start_drag_figure $gid $board %x %y]
    $board bind figure <B1-Motion> \
	[list [namespace current]::drag_figure $gid $board %x %y]
    $board bind figure <ButtonRelease-1> \
	[list [namespace current]::drag_end $gid $board %x %y]

    make_default_position $gid $board
    draw_position $gid $board

    if {$flags($gid,color_on_move) == $flags($gid,our_color)} {
	$board config -cursor ""
    } else {
	$board config -cursor watch
    }
    find_legal_moves $gid $flags($gid,color_on_move)
}

proc chess::draw_position {id board} {
    variable pixmap
    variable position

    $board delete figure

    for {set c 0} {$c < 8} {incr c} {
	for {set r 0} {$r < 8} {incr r} {
	    if {$position($id,$c,$r) != ""} {
		$board create image [center $c $r] \
		    -image $pixmap($position($id,$c,$r)) \
		    -tags [list figure $position($id,$c,$r) [list cr $c $r]]
	    }
	}
    }
}

proc chess::make_default_position {id board} {
    variable position

    for {set c 0} {$c < 8} {incr c} {
	for {set r 0} {$r < 8} {incr r} {
	    set position($id,$c,$r) ""
	}
    }
    for {set c 0} {$c < 8} {incr c} {
	set position($id,$c,1) wp
	set position($id,$c,6) bp
    }
    set c 0
    foreach f {r n b q k b n r} {
	set position($id,$c,0) w$f
	set position($id,$c,7) b$f
	incr c
    }
    set position($id,wk,c) 4
    set position($id,wk,r) 0
    set position($id,bk,c) 4
    set position($id,bk,r) 7
}

proc chess::start_drag_figure {id board x y} {
    variable last_x
    variable last_y
    variable current

    lassign [lindex [lmatch -regexp [$board gettags current] ^cr] 0] cr \
	current(c) current(r)

    set last_x [$board canvasx $x]
    set last_y [$board canvasy $y]
    $board raise current
    $board config -cursor hand2

    highlight_legal_moves $id $board $current(c) $current(r)
}

proc chess::drag_figure {id board x y} {
    variable last_x
    variable last_y
    set x [$board canvasx $x]
    set y [$board canvasy $y]
    $board move current [expr {$x - $last_x}] [expr {$y - $last_y}]
    set last_x $x
    set last_y $y

    $board itemconfigure dst_sq&&square -outline ""
    $board dtag dst_sq
    $board itemconfigure legal&&square -outline blue
    $board addtag dst_sq overlapping $x $y $x $y
    lassign [lindex [lmatch -regexp [$board gettags dst_sq&&background] ^cr] 0] \
	cr c r
    $board addtag dst_sq withtag [list cr $c $r]&&square
    $board itemconfigure dst_sq&&square -outline red
    $board itemconfigure dst_sq&&legal&&square -outline white
}

proc chess::drag_end {id board x y} {
    variable current
    variable flags
    variable allow_illegal

    set x [$board canvasx $x]
    set y [$board canvasy $y]
    $board dtag dst_sq
    $board addtag dst_sq overlapping $x $y $x $y

    lassign [lindex [lmatch \
			 -regexp [$board gettags dst_sq&&background] \
			 ^cr] 0] cr c r
    $board addtag dst_sq withtag [list cr $c $r]&&square

    if {$allow_illegal || $flags($id,color_on_move) == $flags($id,our_color)} {
	do_move $id $board $current(c) $current(r) $c $r ""
    }
    if {$flags($id,color_on_move) == $flags($id,our_color)} {
	$board config -cursor ""
    } else {
	$board config -cursor watch
    }

    $board itemconfigure dst_sq&&square \
	-outline ""
    $board dtag dst_sq
    draw_position $id $board

    unhighlight_legal_moves $id $board
}

proc chess::do_move {id board cf rf ct rt prom} {
    variable flags
    variable position
    variable allow_illegal
    set options "-"

    if {$cf == $ct && $rf == $rt || $ct == "" || $rt == ""} {
	return
    }

    #puts [list $cf $rf $ct $rt]

    if {$allow_illegal || [is_move_legal $id $cf $rf $ct $rt]} {
	if {$position($id,$ct,$rt) != ""} {
	    set options ":"
	}
	set f $position($id,$cf,$rf)
	set position($id,$ct,$rt) $position($id,$cf,$rf)
	set position($id,$cf,$rf) ""

	if {$position($id,$ct,$rt) == "wk"} {
	    set position($id,wk,c) $ct
	    set position($id,wk,r) $rt
	} elseif {$position($id,$ct,$rt) == "bk"} {
	    set position($id,bk,c) $ct
	    set position($id,bk,r) $rt
	}
 	
 	if {[is_white $flags($id,color_on_move)]} {
 	    set p "w"
 	    set opp "b"
 	    set r 0
 	} else {
 	    set p "b"
 	    set opp "w"
 	    set r 7
 	}
 	
	if {$rt == [expr 7 - $r]} {
	    if {$ct == 0} {
		set flags($id,${opp}cq_legal) 0
	    } elseif {$ct == 7} {
		set flags($id,${opp}ck_legal) 0
	    }
	}
 	if {$position($id,$ct,$rt) == "${p}k"} {
 	    set flags($id,${p}ck_legal) 0
 	    set flags($id,${p}cq_legal) 0
 	    if {[expr $ct - $cf] == 2} {
 		set options {0-0}
 		set position($id,5,$r) "${p}r"
 		set position($id,7,$r) ""
 	    } elseif {[expr $ct - $cf] == -2} {
 		set options {0-0-0}
 		set position($id,3,$r) "${p}r"
 		set position($id,0,$r) ""
 	    }
 	} elseif {$position($id,$ct,$rt) == "${p}r"} {
 	    if {($cf == 0) && ($rf == $r)} {
 		set flags($id,${p}cq_legal) 0
 	    } elseif {($cf == 7) && ($rf == $r)} {
 		set flags($id,${p}ck_legal) 0
 	    }
 	}

 	if {$position($id,$ct,$rt) == "${p}p"} {
	    if {(($p == "w") && ($rt == 7)) || \
		(($p == "b") && ($rt == 0))} {
		if {$flags($id,our_color) == $flags($id,color_on_move)} {
		    set promote [promote_pawn $p]
		    if {$promote == -1} {
			set position($id,$ct,$rt) "${p}q"
		    } else {
			set position($id,$ct,$rt) $promote
		    }
		} else {
		    if {$prom != ""} {
			set position($id,$ct,$rt) "${p}$prom"
		    }
		    # If we are here, then it's an error...
		}
		set prom $position($id,$ct,$rt)
		set options [concat $options $prom]
		puts "options $options"
 	    } elseif {$flags($id,enpassant) == "$ct,$rt"} {
 		if {[is_white $flags($id,color_on_move)]} {
 		    set position($id,$ct,4) ""
 		} else {
 		    set position($id,$ct,3) ""
 		}
		set options ":"
 	    } else {
 		if {[expr $rt - $rf] == 2} {
 		    set flags($id,enpassant) $ct,[expr $rt - 1]
 		} elseif {[expr $rt - $rf] == -2} {
 		    set flags($id,enpassant) $ct,[expr $rt + 1]
 		} else {
 		    set flags($id,enpassant) {}
 		}
 	    }
 	}
 	

	if {$flags($id,color_on_move) == $flags($id,our_color)} {
	    send_move $id $cf $rf $ct $rt $prom
	}

 	add_move_to_history $id $cf $rf $ct $rt $f $options

	if {[is_white $flags($id,color_on_move)]} {
	    set flags($id,color_on_move) black
	} else {
	    set flags($id,color_on_move) white
	}
	find_legal_moves $id $flags($id,color_on_move)
    }
}

proc chess::promote_pawn {color} {
    variable pixmap
    variable square_size
    variable line_width
    
    set w .promote
    if {[winfo exists $w]} {
	destroy $w
    }

    Dialog .promote -title "Pawn promotion" \
	-separator 0 -anchor e -default 0

    set select [canvas $w.select \
	-width [expr {($square_size + $line_width) * 4}] \
	-height [expr {($square_size + $line_width)}]]

    pack $select

    set c 0
    foreach fig {q r b n} {
	set img [expr {$c % 2 ? "bf" : "wf"}]
	set x [expr {($square_size + $line_width) * $c}]
	$select create image $x 0 -image $pixmap($img) -anchor nw \
	    -tags [list $fig]
	$select create image $x 0 -image $pixmap($color$fig) -anchor nw \
	    -tags [list $fig]
	incr c
    }

    $select bind q <ButtonRelease-1> \
	[list eval "Dialog::enddialog $w ${color}q"]
    $select bind r <ButtonRelease-1> \
	[list eval "Dialog::enddialog $w ${color}r"]
    $select bind b <ButtonRelease-1> \
	[list eval "Dialog::enddialog $w ${color}b"]
    $select bind n <ButtonRelease-1> \
	[list eval "Dialog::enddialog $w ${color}n"]

    $w draw
}

proc chess::send_move {id cf rf ct rt prom} {
    variable flags

    jlib::send_iq set \
	[jlib::wrapper:createtag turn \
	     -vars [list xmlns games:board \
			type chess \
			id $flags($id,id)] \
	     -subtags [list [make_move_tag $id $cf $rf $ct $rt $prom]]] \
	-to $flags($id,opponent)
    #-command [list [namespace current]::invite_res $jid $id]
}

proc chess::make_move_tag {id cf rf ct rt prom} {
    variable prom_name

    if {$prom == ""} {
	jlib::wrapper:createtag move -vars [list pos "$cf,$rf;$ct,$rt"]
    } else {
	set f [string index $prom 1]
	jlib::wrapper:createtag move -vars [list pos "$cf,$rf;$ct,$rt"] \
	    -subtags [list [jlib::wrapper:createtag promotion -chdata $prom_name($f)]]
    }
}

proc chess::add_move_to_history {id cf rf ct rt f opts} {
    variable flags
    variable piece_name

    if {![info exists flags($id,movenum)]} {
	set flags($id,movenum) 1
    }

    lappend flags($id,history) [list $cf $rf $ct $rt $f $opts]

    set hw $flags($id,hw)
    $hw delete 0.0 end

    set i 1
    foreach {w b} $flags($id,history) {
	$hw insert end "${i}.\t"
	if {$w != {}} {
	    lassign $w cf rf ct rt f opts
	    lassign $opts opt prom
	    if {$prom == ""} {
		set m ""
	    } else {
		set m $piece_name($prom)
	    }
	    if {($opt == "0-0") || ($opt == "0-0-0")} {
		$hw insert end "$opt\t"
	    } else {
		incr rf
		incr rt
		set lf [translit 0-7 a-h $cf]
		set lt [translit 0-7 a-h $ct]
		set n $piece_name($f)
		$hw insert end "$n${lf}$rf${opt}${lt}$rt$m\t"
	    }
	}
	if {$b != {}} {
	    lassign $b cf rf ct rt f opts
	    lassign $opts opt prom
	    if {$prom == ""} {
		set m ""
	    } else {
		set m $piece_name($prom)
	    }
	    if {($opt == "0-0") || ($opt == "0-0-0")} {
		$hw insert end "$opt\n"
	    } else {
		incr rf
		incr rt
		set lf [translit 0-7 a-h $cf]
		set lt [translit 0-7 a-h $ct]
		set n $piece_name($f)
		$hw insert end "$n${lf}$rf${opt}${lt}$rt$m\n"
	    }
	}
	incr i
    }
    $hw see end
    incr flags($id,movenum)
}




proc chess::find_pseudo_legal_moves {id color callback} {
    variable flags
    variable position
    variable moves

    set c [string index $color 0]

    for {set cf 0} {$cf < 8} {incr cf} {
	for {set rf 0} {$rf < 8} {incr rf} {
	    if {[string index $position($id,$cf,$rf) 0] != $c} {
		continue
	    }

	    switch -- $position($id,$cf,$rf) {
		"" { continue }

		wp {
		    set rt [expr $rf + 1]
		    if {$rf < 7 && $position($id,$cf,$rt) == ""} {
			if {$rf == 6} {
			    eval $callback $id $cf $rf $cf $rt promotion
			} else {
			    eval $callback $id $cf $rf $cf $rt
			}
		    }

		    if {$rf == 1 && $position($id,$cf,2) == "" && \
			    $position($id,$cf,3) == ""} {
			eval $callback $id $cf $rf $cf 3
		    }

		    foreach {ct rt} $moves(wpt,$cf,$rf) {
			if {[is_black $position($id,$ct,$rt)] || ($flags($id,enpassant) == "$ct,$rt")} {
			    if {$rf == 6} {
				eval $callback $id $cf $rf $ct $rt promotion
			    } else {
				eval $callback $id $cf $rf $ct $rt
			    }
			}
		    }
		}

		bp {
		    set rt [expr $rf - 1]
		    if {$rf > 0 && $position($id,$cf,$rt) == ""} {
			if {$rf == 1} {
			    eval $callback $id $cf $rf $cf $rt promotion
			} else {
			    eval $callback $id $cf $rf $cf $rt
			}
		    }

		    if {$rf == 6 && $position($id,$cf,5) == "" && \
			    $position($id,$cf,4) == ""} {
			eval $callback $id $cf $rf $cf 4
		    }

		    foreach {ct rt} $moves(bpt,$cf,$rf) {
			if {[is_white $position($id,$ct,$rt)] || \
				$flags($id,enpassant) == "$ct,$rt"} {
			    if {$rf == 1} {
				eval $callback $id $cf $rf $ct $rt promotion
			    } else {
				eval $callback $id $cf $rf $ct $rt
			    }
			}
		    }
		}

		wn -
		bn {
		    foreach {ct rt} $moves(n,$cf,$rf) {
			if {[is_same_color $position($id,$cf,$rf) \
				 $position($id,$ct,$rt)]} \
			    continue
			eval $callback $id $cf $rf $ct $rt
		    }
		}

		wb -
		bb {
		    foreach d {d1 d2 d3 d4} {
			foreach {ct rt} $moves($d,$cf,$rf) {
			    if {[is_same_color $position($id,$cf,$rf) \
				 $position($id,$ct,$rt)]} \
				break
			    eval $callback $id $cf $rf $ct $rt
			    if {$position($id,$ct,$rt) != ""} \
				break
			}
		    }
		}

		wr -
		br {
		    foreach d {h1 h2 v1 v2} {
			foreach {ct rt} $moves($d,$cf,$rf) {
			    if {[is_same_color $position($id,$cf,$rf) \
				$position($id,$ct,$rt)]} \
				break
			    eval $callback $id $cf $rf $ct $rt
			    if {$position($id,$ct,$rt) != ""} \
				break
			}
		    }
		}

		wq -
		bq {
		    foreach d {d1 d2 d3 d4 h1 h2 v1 v2} {
			foreach {ct rt} $moves($d,$cf,$rf) {
			    if {[is_same_color $position($id,$cf,$rf) \
				$position($id,$ct,$rt)]} \
				break
			    eval $callback $id $cf $rf $ct $rt
			    if {$position($id,$ct,$rt) != ""} \
				break
			}
		    }
		}

 		wk {
 		    foreach {ct rt} $moves(k,$cf,$rf) {
 			if {[is_same_color $position($id,$cf,$rf) \
				 $position($id,$ct,$rt)]} \
 			    continue
 			eval $callback $id $cf $rf $ct $rt
 		    }
 		    if {($cf == 4) && ($rf == 0)} {
 			if {$flags($id,wck_legal) && \
				$position($id,5,0) == "" && \
				$position($id,6,0) == ""} {
 			    eval $callback $id 4 0 6 0 kcastling
 			}
 			if {$flags($id,wcq_legal) && \
				$position($id,3,0) == "" && \
				$position($id,2,0) == "" && \
				$position($id,1,0) == ""} {
 			    eval $callback $id 4 0 2 0 qcastling
 			}
 		    }
 		}
 		bk {
 		    foreach {ct rt} $moves(k,$cf,$rf) {
 			if {[is_same_color $position($id,$cf,$rf) \
				 $position($id,$ct,$rt)]} \
 			    continue
 			eval $callback $id $cf $rf $ct $rt
 		    }
 		    if {($cf == 4) && ($rf == 7)} {
 			if {$flags($id,bck_legal) && \
				$position($id,5,7) == "" && \
				$position($id,6,7) == ""} {
 			    eval $callback $id 4 7 6 7 kcastling
 			}
 			if {$flags($id,bcq_legal) && \
				$position($id,3,7) == "" && \
				$position($id,2,7) == "" && \
				$position($id,1,7) == ""} {
 			    eval $callback $id 4 7 2 7 qcastling
			}
		    }
		}
	    }
	}
    }

#    puts $moves($id)
}


proc chess::find_legal_moves {id color} {
    variable moves

    set moves($id) {}
    find_pseudo_legal_moves $id $color check_legal_callback
}

proc chess::check_legal_callback {id cf rf ct rt {opt ""}} {
    variable moves

    if {![test_check $id $cf $rf $ct $rt]} {
	lappend moves($id) [list $cf $rf $ct $rt $opt]
    }
}

proc chess::test_check {id cf rf ct rt {opt ""}} {
    variable flags
    variable position
    variable checks
    variable moves

    set enpassantback ""
    set enpassantx ""
    set enpassanty ""
    if {[is_white $flags($id,color_on_move)]} {
	set color black
	set f wk
	set of bk
	if {($rf == 4) && ($position($id,$cf,$rf) == "wp") && ($flags($id,enpassant) == "$ct,$rt")} {
	    set enpassantback "bp"
	    set enpassantx $ct
	    set enpassanty [expr $rt - 1]
	    set position($id,$enpassantx,$enpassanty) ""
	}
    } else {
	set color white
	set f bk
	set of wk
	if {($rf == 3) && ($position($id,$cf,$rf) == "bp") && ($flags($id,enpassant) == "$ct,$rt")} {
	    set enpassantback "wp"
	    set enpassantx $ct
	    set enpassanty [expr $rt + 1]
	    set position($id,$enpassantx,$enpassanty) ""
	}
    }
    set checks 0

    set back $position($id,$ct,$rt)
    set position($id,$ct,$rt) $position($id,$cf,$rf)
    set position($id,$cf,$rf) ""
    if {$position($id,$ct,$rt) == "wk"} {
	set position($id,wk,c) $ct
	set position($id,wk,r) $rt
    } elseif {$position($id,$ct,$rt) == "bk"} {
	set position($id,bk,c) $ct
	set position($id,bk,r) $rt
    }

#    for {set c 0} {$c < 8} {incr c} { # Probably it's better to store kings positions
#	for {set r 0} {$r < 8} {incr r} {
#	    if {$position($id,$c,$r) == $f} {
#		set kc $c
#		set kr $r
#	    } elseif {$position($id,$c,$r) == $of} {
#		set okc $c
#		set okr $r
#	    }
#	}
#    }
    set kc  $position($id,$f,c)
    set kr  $position($id,$f,r)
    set okc $position($id,$of,c)
    set okr $position($id,$of,r)


    if {$opt == "kcastling"} {
 	set kcs [list 4 5 6]
    } elseif {$opt == "qcastling"} {
 	set kcs [list 4 3 2]
    } else {
 	set kcs [list $kc]
    }

    foreach c $kcs {
 	incr checks [test_figures $id $color $c $kr]
 	incr checks [test_pawns $id $color $c $kr]
 	if {[info exists okc]} {
 	    set dx [expr abs($c-$okc)]
 	    set dy [expr abs($kr-$okr)]
 	    if {($dx <= 1) && ($dy <= 1)} {
     		incr checks
 	    }
 	}
    }


    set position($id,$cf,$rf) $position($id,$ct,$rt)
    set position($id,$ct,$rt) $back
    if {$position($id,$cf,$rf) == "wk"} {
	set position($id,wk,c) $cf
	set position($id,wk,r) $rf
    } elseif {$position($id,$cf,$rf) == "bk"} {
	set position($id,bk,c) $cf
	set position($id,bk,r) $rf
    }
    if {$enpassantback != ""} {
	set position($id,$enpassantx,$enpassanty) $enpassantback 
    }

    return $checks
}

proc chess::test_pawns {id color c r} {
    variable position
    variable moves

    if {$color == "white"} {
	foreach {x y} $moves(bpt,$c,$r) {
	    if {$position($id,$x,$y) == "wp"} {
	    	return 1
	    }
	}
    } else {
	foreach {x y} $moves(wpt,$c,$r) {
#	#    puts -[list $position($id,$x,$y) $x $y]
	    if {$position($id,$x,$y) == "bp"} {
	    	return 1
	    }
	}
    }
    return 0
}

proc chess::test_figures {id color c r} {
    variable position
    variable moves

    if {$color == "white"} {
    	foreach i {d1 d2 d3 d4} {
	    foreach {x y} $moves($i,$c,$r) {
    		switch -- $position($id,$x,$y) {
		    "" {continue}
		    wq -
		    wb {return 1}
		    default {break}
		}
	    }
	}
    	foreach i {h1 h2 v1 v2} {
	    foreach {x y} $moves($i,$c,$r) {
    		switch -- $position($id,$x,$y) {
		    "" {continue}
		    wq -
		    wr {return 1}
		    default {break}
		}
	    }
	}
	foreach {x y} $moves(n,$c,$r) {
    	    switch -- $position($id,$x,$y) {
		wn {return 1}
		default {continue}
	    }
	}
    } else {
    	foreach i {d1 d2 d3 d4} {
	    foreach {x y} $moves($i,$c,$r) {
    		switch -- $position($id,$x,$y) {
		    "" {continue}
		    bq -
		    bb {return 1}
		    default {break}
		}
	    }
	}
    	foreach i {h1 h2 v1 v2} {
	    foreach {x y} $moves($i,$c,$r) {
    		switch -- $position($id,$x,$y) {
		    "" {continue}
		    bq -
		    br {return 1}
		    default {break}
		}
	    }
	}
	foreach {x y} $moves(n,$c,$r) {
    	    switch -- $position($id,$x,$y) {
		bn {return 1}
		default {continue}
	    }
	}
    }
    return 0
}

proc chess::test_check_callback {c r id cf rf ct rt {opt ""}} {
    variable checks
    if {$c == $ct && $r == $rt} {
	incr checks
    }
}



proc chess::is_move_legal {id cf rf ct rt} {
    variable moves
    expr {[lmatch -regexp $moves($id) ^[list $cf $rf $ct $rt]] != {}}
}


proc chess::highlight_legal_moves {id board cf rf} {
    variable moves

    foreach move [lmatch -regexp $moves($id) ^[list $cf $rf]] {
	lassign $move cft rft ct rt
	$board addtag legal withtag [list cr $ct $rt]&&square

    }
    $board itemconfigure legal&&square -outline blue
}

proc chess::unhighlight_legal_moves {id board} {
    foreach sq [$board find withtag legal&&square] {
	$board itemconfigure $sq \
	    -outline [$board itemcget $sq -fill]
    }
    $board dtag legal
}


proc chess::is_white {f} {
    expr {[string index $f 0] == "w"}
}

proc chess::is_black {f} {
    expr {[string index $f 0] == "b"}
}

proc chess::is_same_color {f1 f2} {
    expr {[string index $f1 0] == [string index $f2 0]}
}


proc chess::add_user_menu_item {m jids} {
#    roster::add_menu_item $m "Chess board" \
#	[list eval [namespace current]::start_play \$curuser "" white] $jids
    roster::add_menu_item $m "Chess" \
	[list eval [namespace current]::invite_dialog \$curuser] $jids
}

hook::add roster_create_user_menu_hook \
    [namespace current]::chess::add_user_menu_item

proc chess::add_groupchat_user_menu_item {m} {
    $m add command -label "Chess" \
	-command [list eval [namespace current]::invite_dialog \$curgroupuser]
}

hook::add roster_create_groupchat_user_menu_hook \
    [namespace current]::chess::add_groupchat_user_menu_item


chess::load_pixmaps
chess::calc_moves



proc iq_games_board_create {from child} {
    jlib::wrapper:splitxml $child tag vars isempty chdata children

    switch -- [jlib::wrapper:getattr $vars type] {
	chess {
	    return [list result [[namespace current]::chess::invited_dialog \
				     $from [jlib::wrapper:getattr $vars id]]]
	}
	default {
	    return [list error 501 "Not Implemented"]
	}
    }
    return ""
}

iq::register_handler set create games:board \
    [namespace current]::iq_games_board_create



proc iq_games_board_turn {from child} {
    jlib::wrapper:splitxml $child tag vars isempty chdata children

    switch -- [jlib::wrapper:getattr $vars type] {
	chess {
	    return [list result [[namespace current]::chess::turn_recv \
				     $from [jlib::wrapper:getattr $vars id] \
				     $children]]
	}
	default {
	    return [list error 501 "Not Implemented"]
	}
    }
    return ""
}

iq::register_handler set turn games:board \
    [namespace current]::iq_games_board_turn

