# $Id: whiteboard.tcl,v 1.2 2002/10/05 19:18:48 alexey Exp $

uplevel #0 source [file join [file dirname [info script]] svgrender.tcl]

namespace eval wb {}


proc wb::add_conference_menu_items {m jid} {
    $m add command -label Whiteboard \
	-command [list [namespace current]::open_wb $jid]
}

hook::add chat_create_conference_menu_hook \
    [namespace current]::wb::add_conference_menu_items


proc wb::open_wb {jid} {

    set w [win_id whiteboard $jid]
    if {[winfo exists $w]} {
	return
    }

    add_win $w -title "$jid whiteboard" -tabtitle "$jid whiteboard" \
	-class Whiteboard \
	-raisecmd [list tab_set_updated $w]


    set c [canvas $w.c -background white]
    pack $c -side right -fill both -expand yes

    set tb [frame $w.tb]
    pack $tb -side left -fill y

    set tbpolyline [radiobutton $w.tb.line -text PolyLine \
			-variable [namespace current]::tool($jid) \
			-value polyline \
			-command [list [namespace current]::line_bind $c $jid]]
    
    set tbfreehand [radiobutton $w.tb.freehand -text FreeHand \
			-variable [namespace current]::tool($jid) \
			-value freehand \
			-command [list [namespace current]::freehand_bind \
				      $c $jid]]

    pack $tbpolyline $tbfreehand -anchor w

    button $w.tb.clear -text Clear \
	-command [list [namespace current]::send_clear $jid]
    pack $w.tb.clear -side bottom

    canvas $w.tb.color -background \#000000 -height 0.5c -width 1
    button $w.tb.selcol -text "Color" \
	-command [list [namespace current]::select_color \
		      $w.tb.selcol $w.tb.color]
    pack $w.tb.selcol $w.tb.color -side bottom

    SpinBox $w.tb.width -width 4 -range {1 100} \
	-textvariable [namespace current]::width($jid)
    pack $w.tb.width -side bottom

    trace variable [namespace current]::width($jid) w \
	[list [namespace current]::change_width \
	     $w.tb.color [namespace current]::width($jid)]

    variable tool
    set tool($jid) polyline
    line_bind $c $jid
}

proc wb::select_color {but col} {
    set color [SelectColor::menu $col.color [list below $but] \
		   -color [$col cget -background]]

    if {[string length $color]} {
        $col configure -background $color
    }
}

proc wb::get_color {jid} {
    set w [win_id whiteboard $jid]
    $w.tb.color cget -background
}

proc wb::change_width {col widthvar args} {
    set width [set $widthvar]
    $col configure -width $width
}

proc wb::get_width {jid} {
    set w [win_id whiteboard $jid]
    $w.tb.color cget -width
}

###############################################################################

proc wb::send_svg {jid tag} {
    jlib::send_msg $jid -type groupchat \
	-xlist [list [jlib::wrapper:createtag x \
			  -vars {xmlns tkabber:whiteboard} \
			  -subtags [list [jlib::wrapper:createtag svg \
					      -subtags [list $tag]]]]]
}

proc wb::send_clear {jid} {
    jlib::send_msg $jid -type groupchat \
	-xlist [list [jlib::wrapper:createtag x \
			  -vars {xmlns tkabber:whiteboard} \
			  -subtags [list [jlib::wrapper:createtag clear]]]]
}

proc wb::handle_wb {chatid from type body x} {
    foreach xelem $x {
	jlib::wrapper:splitxml $xelem tag vars isempty chdata children
	
	if {[cequal [jlib::wrapper:getattr $vars xmlns] tkabber:whiteboard]} {
	    open_wb $chatid
	    set w [win_id whiteboard $chatid]
	    foreach child $children {
		parse_item $chatid $child
	    }
	    tab_set_updated $w 1 message
	}
    }
}
hook::add draw_message_hook [namespace current]::wb::handle_wb 1

proc wb::parse_item {jid item} {
    set w [win_id whiteboard $jid]
    jlib::wrapper:splitxml $item tag vars isempty chdata children

    switch -- $tag {
	svg {
	    foreach child $children {
		svg::parse_svg_item $w.c $child
	    }
	}
	clear {
	    $w.c delete all
	}
    }
}

###############################################################################
# Line

proc wb::line_bind {c jid} {
    bind $c <ButtonPress-1> \
	[list [namespace current]::line_b1 [double% $c] [double% $jid] %x %y]
    bind $c <B1-Motion> {}
    bind $c <Motion> [list [namespace current]::line_b1m [double% $c] %x %y]
    bind $c <ButtonRelease-1> {}
    bind $c <Button-3> [list [namespace current]::line_b3 \
			    [double% $c] [double% $jid]]
}

proc wb::line_b1 {c jid x y} {
    variable line

    set x [$c canvasx $x]
    set y [$c canvasy $y]
    #puts "$x $y"

    if {[info exists line(drawed)]} {
	lappend line(coords) $x $y

	catch {$c delete $line(temp)}
	set line(temp) [eval $c create line $line(coords) $line(options)]
    } else {
	set line(drawed) 1
	set line(coords) "$x $y"
	set line(options) [list -fill [get_color $jid] \
			       -width [get_width $jid] \
			       -joinstyle miter]
    }
}

proc wb::line_b1m {c x y} {
    variable line

    set x [$c canvasx $x]
    set y [$c canvasy $y]

    if {[info exists line(drawed)]} {
	#lappend line(coords) $x $y

	catch {$c delete $line(temp)}
	set line(temp) [eval $c create line $line(coords) $x $y $line(options)]
    }
}


proc wb::line_b3 {c jid} {
    variable line

    catch {
	unset line(drawed)

	catch {$c delete $line(temp)}
	set line(temp) [eval $c create line $line(coords) $line(options)]

	$c delete $line(temp)

	set points {}
	foreach {x y} $line(coords) {
	    lappend points $x,$y
	}
	lappend vars points $points
	if {[set color [get_color $jid]] != "#000000"} {
	    lappend vars stroke $color
	}
	if {[set width [get_width $jid]] != 1} {
	    lappend vars stroke-width $width
	}

	send_svg $jid [jlib::wrapper:createtag polyline \
			   -vars $vars]

	set line(coords) {}
	set line(temp) {}
    }
}

###############################################################################

###############################################################################
# Freehand

proc wb::freehand_bind {c jid} {
    bind $c <ButtonPress-1> \
	[list [namespace current]::freehand_b1p \
	     [double% $c] [double% $jid] %x %y]
    bind $c <B1-Motion> [list [namespace current]::freehand_b1m \
			     [double% $c] %x %y]
    bind $c <ButtonRelease-1> \
	[list [namespace current]::freehand_b1r [double% $c] [double% $jid]]
    bind $c <Button-3> {}
}

proc wb::freehand_b1p {c jid x y} {
    variable line

    set x [$c canvasx $x]
    set y [$c canvasy $y]
    #puts "$x $y"

    if {0 && [info exists line(drawed)]} {
	lappend line(coords) $x $y

	catch {$c delete $line(temp)}
	set line(temp) [eval $c create line $line(coords)]
    } else {
	set line(drawed) 1
	set line(coords) "$x $y"
	set line(options) [list -fill [get_color $jid] \
			       -width [get_width $jid] \
			       -joinstyle miter]
    }
}

proc wb::freehand_b1m {c x y} {
    variable line

    set x [$c canvasx $x]
    set y [$c canvasy $y]

    if {[info exists line(drawed)]} {
	lappend line(coords) $x $y

	catch {$c delete $line(temp)}
	set line(temp) [eval $c create line $line(coords) $x $y $line(options)]
    }
}


proc wb::freehand_b1r {c jid} {
    variable line

    catch {
	unset line(drawed)

	catch {$c delete $line(temp)}
	set line(temp) [eval $c create line $line(coords) $line(options)]

	$c delete $line(temp)

	set points {}
	foreach {x y} $line(coords) {
	    lappend points $x,$y
	}
	lappend vars points $points
	if {[set color [get_color $jid]] != "#000000"} {
	    lappend vars stroke $color
	}
	if {[set width [get_width $jid]] != 1} {
	    lappend vars stroke-width $width
	}

	send_svg $jid [jlib::wrapper:createtag polyline \
			   -vars $vars]

	set line(coords) {}
	set line(temp) {}
    }
}

###############################################################################

